package com.jjt.biz.service.impl;

import com.jjt.biz.domain.AlarmRecord;
import com.jjt.biz.domain.BizObjMetrics;
import com.jjt.biz.mapper.AlarmRecordMapper;
import com.jjt.biz.service.IAlarmRecordService;
import com.jjt.biz.vo.BizAccessVO;
import com.jjt.biz.vo.BizTypeVO;
import com.jjt.common.utils.DateUtils;
import com.jjt.common.utils.IntervalUtil;
import com.jjt.common.utils.StringUtils;
import com.jjt.push.service.IPushConfigService;
import com.jjt.system.service.ISysDictDataService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 告警记录Service业务层处理
 *
 * @author jjt
 * @date 2024-08-08
 */
@Service
public class AlarmRecordServiceImpl implements IAlarmRecordService {
    @Resource
    private AlarmRecordMapper alarmRecordMapper;
    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource
    private ISysDictDataService sysDictDataService;
    @Resource
    private IPushConfigService pushConfigService;

    /**
     * 查询告警记录
     *
     * @param alarmId 告警记录主键
     * @return 告警记录
     */
    @Override
    public AlarmRecord selectAlarmRecordByAlarmId(Long alarmId) {
        return alarmRecordMapper.selectAlarmRecordByAlarmId(alarmId);
    }

    /**
     * 查询告警记录列表
     *
     * @param alarmRecord 告警记录
     * @return 告警记录
     */
    @Override
    public List<AlarmRecord> selectAlarmRecordList(AlarmRecord alarmRecord) {
        return alarmRecordMapper.selectAlarmRecordList(alarmRecord);
    }

    /**
     * 新增告警记录
     *
     * @param alarmRecord 告警记录
     * @return 结果
     */
    @Override
    public int insertAlarmRecord(AlarmRecord alarmRecord) {
        alarmRecord.setCreateTime(DateUtils.getNowDate());
        return alarmRecordMapper.insertAlarmRecord(alarmRecord);
    }

    /**
     * 修改告警记录
     *
     * @param alarmRecord 告警记录
     * @return 结果
     */
    @Override
    public int updateAlarmRecord(AlarmRecord alarmRecord) {
        alarmRecord.setUpdateTime(DateUtils.getNowDate());
        return alarmRecordMapper.updateAlarmRecord(alarmRecord);
    }

    /**
     * 批量删除告警记录
     *
     * @param alarmIds 需要删除的告警记录主键
     * @return 结果
     */
    @Override
    public int deleteAlarmRecordByAlarmIds(Long[] alarmIds) {
        return alarmRecordMapper.deleteAlarmRecordByAlarmIds(alarmIds);
    }

    /**
     * 删除告警记录信息
     *
     * @param alarmId 告警记录主键
     * @return 结果
     */
    @Override
    public int deleteAlarmRecordByAlarmId(Long alarmId) {
        return alarmRecordMapper.deleteAlarmRecordByAlarmId(alarmId);
    }

    /**
     * 告警等级排行
     *
     * @param alarmRecord 对象
     * @return 结果
     */
    @Override
    public List<Map<String, Object>> levelSort(AlarmRecord alarmRecord) {
        return alarmRecordMapper.levelSort(alarmRecord);
    }

    /**
     * 按小时告警趋势
     *
     * @param alarmRecord 对象
     * @return 结果
     */
    @Override
    public List<Map<String, Object>> listHours(AlarmRecord alarmRecord) {
        return alarmRecordMapper.listHours(alarmRecord);
    }

    /**
     * 处理告警和事件
     *
     * @param om        指标对象
     * @param recordMap 正在告警的列表
     * @param date      时间
     * @return 结果
     */
    @Override
    public Map<String, AlarmRecord> process(BizObjMetrics om, Map<Long, AlarmRecord> recordMap, Date date) {
        Map<String, AlarmRecord> result = new HashMap<>(16);
        // 需要查询是否已经有告警记录
        // 1.当前没有告警，但之前有未结束的告警。
        // 2.如果该指标有告警，且告警等级相同，则不用管
        // 3.如果该指标有告警，但告警等级不同，则上一次告警结束，新增一条告警。
        // 4.如果之前没有告警，当前告警，则新增告警。
        AlarmRecord record = recordMap.get(om.getObjMetricsId());
        Map<String, String> pair = getAlarmLevel(om.getDValue().floatValue(), om);
        if (pair == null) {
            //如果没有告警，且之前有告警记录，则结束之前告警
            if (record != null) {
                record.setAlarmType("2");
                record.setEndTime(date);
                result.put("update", record);
            }
            return result;
        }
        String alarmLevel = pair.get("key");

        if (record != null) {
            if (alarmLevel.equals(record.getAlarmLevel())) {
                //如果有相同告警，则本次不处理
                return result;
            } else {
                //有告警记录，但告警等级不一样
                //结束上一次告警
                record.setAlarmType("2");
                record.setEndTime(date);
                result.put("update", record);

//                updateAlarmRecord(record);
            }
        }
        record = new AlarmRecord();
        record.setObjId(om.getObjId());
        record.setObjName(om.getObjName());
        record.setObjMetricsId(om.getObjMetricsId());
        record.setMetricsName(om.getMetricsName());
        record.setAlarmLevel(alarmLevel);
        record.setAlarmValue(om.getDValue());
        record.setAlarmType("1");
        record.setAlarmTime(date);
        record.setRemark(pair.get("value"));
        record.setCreateTime(new Date());
        result.put("add", record);
        pushConfigService.push(record);
        return result;
//        insertAlarmRecord(record);
    }

    /**
     * 通过对象ID查询当前正在告警的列表,并转换成map
     *
     * @param objId 对象ID
     * @return 结果
     */
    @Override
    public Map<Long, AlarmRecord> selectAlarmRecordListCurr(Long objId) {
        AlarmRecord search = new AlarmRecord();
        search.setObjId(objId);
        search.setAlarmType("1");
        List<AlarmRecord> list = selectAlarmRecordList(search);
        return list.stream().collect(Collectors.toMap(AlarmRecord::getObjMetricsId, Function.identity(), (existing, replacement) -> existing));
    }

    /**
     * 业务类型统计
     *
     * @param start 开始时间
     * @param end   结束时间
     * @return 结果
     */
    @Override
    public List<BizTypeVO> listBizType(String start, String end) {
        List<BizTypeVO> list = new ArrayList();
        String sql = "SELECT b.obj_type,COUNT(*) num FROM alarm_record a,biz_obj b WHERE a.obj_id=b.obj_id";
        Vector<Object> v = new Vector<>();
        if (start != null && end != null) {
            sql += " AND a.alarm_time BETWEEN ? AND ?";
            v.add(start);
            v.add(end + " 23:59:59");
        }
        sql += " GROUP BY b.obj_type";
        List<Map<String, Object>> results = jdbcTemplate.queryForList(sql, v.toArray());
        results.forEach(map -> {
            BizTypeVO ls = new BizTypeVO();

            String obj_type = (String) map.get("obj_type");
            ls.setId(obj_type);
            ls.setName(sysDictDataService.selectDictLabel("biz_type", obj_type));
            ls.setNum((Long) map.get("num"));
            list.add(ls);
        });

        list.sort(Comparator.comparing(BizTypeVO::getNum));
        return list;
    }

    /**
     * 业务组件统计
     *
     * @param type  业务类型
     * @param start 开始时间
     * @param end   结束时间
     * @return 结果
     */
    @Override
    public List<BizTypeVO> listBizObj(String type, String start, String end) {
        List<BizTypeVO> list = new ArrayList();
        String sql = "SELECT b.obj_id,b.obj_name,COUNT(*) num FROM alarm_record a,biz_obj b WHERE a.obj_id=b.obj_id";
        Vector<Object> v = new Vector<>();
        if (StringUtils.isNotEmpty(type)) {
            sql += " AND b.obj_type=?";
            v.add(type);
        }
        if (start != null && end != null) {
            sql += " AND a.alarm_time BETWEEN ? AND ?";
            v.add(start);
            v.add(end + " 23:59:59");
        }
        sql += " GROUP BY b.obj_id,b.obj_name";
        List<Map<String, Object>> results = jdbcTemplate.queryForList(sql, v.toArray());
        results.forEach(map -> {
            BizTypeVO ls = new BizTypeVO();
            String obj_name = (String) map.get("obj_name");
            int obj_id = (int) map.get("obj_id");
            ls.setId(obj_id + "");
            ls.setName(obj_name);
            ls.setNum((Long) map.get("num"));
            list.add(ls);
        });

        list.sort(Comparator.comparing(BizTypeVO::getNum));
        return list;
    }

    /**
     * 组件压力排名
     *
     * @param modelId 模型ID
     * @param start   开始时间
     * @param end     结束时间
     * @return 结果
     */
    @Override
    public List<BizAccessVO> objAccess(Long modelId, String start, String end) {
        List<BizAccessVO> list = new ArrayList<>();

        String sql = "SELECT b.OBJ_ID,c.OBJ_NAME,c.OBJ_ADDR FROM biz_model_detail b,biz_obj c WHERE b.obj_id=c.obj_id AND c.OBJ_TYPE=1";
        if (modelId != null) {
            sql += " AND b.model_id=" + modelId;
        }
        sql += " GROUP BY b.OBJ_ID,c.obj_name";

        List<Map<String, Object>> objList = jdbcTemplate.queryForList(sql);
        objList.forEach(map -> {
            BizAccessVO vo = new BizAccessVO();
            Integer objId = (Integer) map.get("OBJ_ID");
            String objName = (String) map.get("OBJ_NAME");
            String code = (String) map.get("OBJ_ADDR");
            Vector<Object> v = new Vector<>();
            v.add(objId);
            String totSql = "SELECT IFNULL(SUM(D_VALUE),0) FROM biz_obj_metrics_data " +
                    "WHERE obj_metrics_id IN(" +
                    "SELECT obj_metrics_id FROM biz_obj_metrics" +
                    " WHERE metrics_id =(SELECT metrics_id FROM metrics_def WHERE metrics_code='pp.tot')" +
                    " AND obj_id =?)";
            String errSql = "SELECT IFNULL(SUM(D_VALUE),0) FROM biz_obj_metrics_data " +
                    "WHERE obj_metrics_id IN(" +
                    "SELECT obj_metrics_id FROM biz_obj_metrics" +
                    " WHERE metrics_id =(SELECT metrics_id FROM metrics_def WHERE metrics_code='pp.error')" +
                    " AND obj_id =?)";

            if (start != null && end != null) {
                totSql += " AND CREATE_TIME BETWEEN ? AND ?";
                errSql += " AND CREATE_TIME BETWEEN ? AND ?";
                v.add(start);
                v.add(end + " 23:59:59");
            }

            Long access = jdbcTemplate.queryForObject(totSql, Long.class, v.toArray());
            Long num = jdbcTemplate.queryForObject(errSql, Long.class, v.toArray());

            vo.setModelName(objName);
            vo.setAccess(access);
            vo.setCode(code);
            vo.setNum(num);
            list.add(vo);
        });
        return list;
    }

    /**
     * 查询是否正在告警？
     *
     * @param metricsId 指标对象
     * @return 结果
     */
//    private AlarmRecord exists(Long metricsId) {
//        AlarmRecord search = new AlarmRecord();
//        search.setObjMetricsId(metricsId);
//        search.setAlarmType("1");
//
//        List<AlarmRecord> list = selectAlarmRecordList(search);
//        if (list.size() > 0) {
//            return list.get(0);
//        } else {
//            return null;
//        }
//    }

    /**
     * 获取告警等级
     *
     * @param value         值
     * @param bizObjMetrics 指标对象
     * @return
     */
    private Map<String, String> getAlarmLevel(Float value, BizObjMetrics bizObjMetrics) {
        Map<String, String> result = new HashMap<>();
        if (StringUtils.isNotEmpty(bizObjMetrics.getAlarmLow()) && IntervalUtil.inNumRange(value, bizObjMetrics.getAlarmLow())) {
            result.put("key", "low");
            result.put("value", bizObjMetrics.getAlarmLow());
            return result;
        } else if (StringUtils.isNotEmpty(bizObjMetrics.getAlarmMid()) && IntervalUtil.inNumRange(value, bizObjMetrics.getAlarmMid())) {
            result.put("key", "mid");
            result.put("value", bizObjMetrics.getAlarmMid());
            return result;
        } else if (StringUtils.isNotEmpty(bizObjMetrics.getAlarmHigh()) && IntervalUtil.inNumRange(value, bizObjMetrics.getAlarmHigh())) {
            result.put("key", "high");
            result.put("value", bizObjMetrics.getAlarmHigh());
            return result;
        }
        return null;
    }


}
