package com.jjt.biz.service.impl;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.http.Method;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.jjt.biz.service.IPinpointService;
import com.jjt.biz.vo.AgentVO;
import com.jjt.common.core.redis.RedisCache;
import com.jjt.common.utils.http.HttpUtils;
import com.jjt.system.service.ISysConfigService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 告警记录Service业务层处理
 *
 * @author jjt
 * @date 2024-08-08
 */
@Service
public class PinpointServiceImpl implements IPinpointService {
    @Resource
    private ISysConfigService configService;
    @Resource
    private RedisCache redisCache;

    /**
     * 获取接口地址
     *
     * @return 接口地址
     */
    private String baseUri() {
        return configService.selectConfigByKey("pp.url");
    }

    /**
     * 获取所有应用列表
     *
     * @return 结果
     */
    @Override
    public Map<String, String> getApps() {
//        Map<String, String> appsMap = redisCache.getCacheMap(CacheConstants.PP_APPS_MAP);
//        if (appsMap == null || appsMap.isEmpty()) {
//            appsMap = new HashMap<>(16);
//            String uri = baseUri() + "/applications.pinpoint";
//            String result = HttpUtils.sendGet(uri);
//            JSONArray jsonArray = JSONArray.parseArray(result);
//            for (int i = 0; i < jsonArray.size(); i++) {
//                JSONObject obj = jsonArray.getJSONObject(i);
//                appsMap.put(obj.getString("applicationName"), obj.getString("serviceType"));
//            }
//            redisCache.setCacheMap(CacheConstants.PP_APPS_MAP, appsMap);
//        }
//        return appsMap;
        //先不做缓存，其他地方要用。
        Map<String, String> appsMap = new HashMap<>(16);
        String uri = baseUri() + "/applications.pinpoint";
        String result = HttpUtils.sendGet(uri);
        JSONArray jsonArray = JSONArray.parseArray(result);
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject obj = jsonArray.getJSONObject(i);
            appsMap.put(obj.getString("applicationName"), obj.getString("serviceType"));
        }
        return appsMap;
    }

    /**
     * 链路查询
     *
     * @param name appName
     * @param type serviceType
     * @param st   开始时间
     * @param ed   结束时间
     * @return 结果
     */
    @Override
    public JSONObject link(String name, String type, LocalDateTime st, LocalDateTime ed) {
        long end = ed.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
        long start = st.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
        String uri = baseUri() + "/getServerMapDataV2.pinpoint?applicationName=%s&serviceTypeName=%s&from=%s&to=%s&calleeRange=1&callerRange=1&wasOnly=false&bidirectional=false&useStatisticsAgentState=false";
        uri = String.format(uri, name, type, start, end);
        String result = HttpUtils.sendGet(uri);
        return JSONObject.parseObject(result);
    }

    /**
     * tps查询
     * "min","max", "avg", "sum" 取max
     * 返回的一组数据，取最大的一条数据
     *
     * @param agentId appName
     * @param st      开始时间
     * @param ed      结束时间
     * @return 结果
     */
    @Override
    public Float tps(String agentId, LocalDateTime st, LocalDateTime ed) {
        long end = ed.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
        long start = st.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
        String uri = baseUri() + "/getAgentStat/transaction/chart.pinpoint?agentId=%s&from=%s&to=%s&sampleRate=1";
        uri = String.format(uri, agentId, start, end);

        String result = HttpUtils.sendGet(uri);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Float tps = 0F;
        try {
            JSONArray array = jsonObject.getJSONObject("charts").getJSONObject("y").getJSONArray("TPS_TOTAL");
            for (int i = 0; i < array.size(); i++) {
                JSONArray arr = array.getJSONArray(i);
                Float v = arr.getFloat(1);
                if (v != 0F && v > tps) {
                    tps = v;
                }
            }
        } catch (Exception ignored) {
        }
        return tps;
    }

    /**
     * 打开文件数量查询
     *
     * @param agentId appName
     * @param st      开始时间
     * @param ed      结束时间
     * @return 结果
     */
    @Override
    public int openFile(String agentId, LocalDateTime st, LocalDateTime ed) {
        long end = ed.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
        long start = st.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
        String uri = baseUri() + "/getAgentStat/fileDescriptor/chart.pinpoint?agentId=%s&from=%s&to=%s&sampleRate=1";
        uri = String.format(uri, agentId, start, end);

        String result = HttpUtils.sendGet(uri);
        JSONObject jsonObject = JSONObject.parseObject(result);
        int num = 0;
        try {
            JSONArray array = jsonObject.getJSONObject("charts").getJSONObject("y").getJSONArray("OPEN_FILE_DESCRIPTOR_COUNT");
            for (int i = 0; i < array.size(); i++) {
                JSONArray arr = array.getJSONArray(i);
                int v = arr.getInteger(1);
                if (v != 0 && v > num) {
                    num = v;
                }
            }
        } catch (Exception ignored) {

        }
        return num;
    }

    /**
     * jvm内存使用情况
     *
     * @param agentId appName
     * @param st      开始时间
     * @param ed      结束时间
     * @return 结果
     */
    @Override
    public Map<String, Object> jvmMemory(String agentId, LocalDateTime st, LocalDateTime ed) {
        long end = ed.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
        long start = st.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
        String uri = baseUri() + "/getAgentStat/jvmGc/chart.pinpoint?agentId=%s&from=%s&to=%s&sampleRate=1";
        uri = String.format(uri, agentId, start, end);

        String result = HttpUtils.sendGet(uri);
        JSONObject jsonObject = JSONObject.parseObject(result);
        float usage = 0;
        int gcTimes = 0;
        try {
            JSONArray heapUsed = jsonObject.getJSONObject("charts").getJSONObject("y").getJSONArray("JVM_MEMORY_HEAP_USED");
            JSONArray heapMax = jsonObject.getJSONObject("charts").getJSONObject("y").getJSONArray("JVM_MEMORY_HEAP_MAX");
            JSONArray noHeapUsed = jsonObject.getJSONObject("charts").getJSONObject("y").getJSONArray("JVM_MEMORY_HEAP_MAX");
            JSONArray noHeapMax = jsonObject.getJSONObject("charts").getJSONObject("y").getJSONArray("JVM_MEMORY_HEAP_MAX");
            JSONArray gc = jsonObject.getJSONObject("charts").getJSONObject("y").getJSONArray("JVM_GC_OLD_COUNT");

            float used = 0F;
            float max = 0F;
            if (heapUsed.size() > 0) {
                for (int i = 0; i < heapUsed.size(); i++) {
                    float v = heapUsed.getJSONArray(i).getFloat(1);
                    int gcV = gc.getJSONArray(i).getInteger(1);
                    if (gcV > 0) {
                        //如果有gc则增加
                        gcTimes += gcV;
                    }
                    if (v != 0f && v > used) {
                        used = v;
                        max = heapMax.getJSONArray(i).getFloat(1);
                    }
                }
                usage = BigDecimal.valueOf(used).divide(BigDecimal.valueOf(max), 2, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
            }
        } catch (Exception ignored) {
        }
        Map<String, Object> map = new HashMap<>(16);
        map.put("usage", usage);
        map.put("gc", gcTimes);
        return map;
    }

    /**
     * 清理失效agent
     */
    @Override
    public void cleanAgent() {
        String uri = baseUri() + "/admin/removeInactiveAgents.pinpoint";
        Map<String, Object> map = new HashMap<>(16);
        map.put("password", "admin");
        HttpRequest request = HttpUtil.createGet(uri);
        request.setMethod(Method.GET);
        request.form(map);
        request.setConnectionTimeout(2000);
        try (HttpResponse execute = request.execute()) {
            if (!execute.isOk()) {
                throw new RuntimeException("status:" + execute.getStatus() + "\tres:" + execute.body());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 删除agent
     *
     * @param applicationName 应用名称
     * @param agentId         应用ID
     */
    @Override
    public void removeAgent(String applicationName, String agentId) {
        String uri = baseUri() + "/admin/removeAgentId.pinpoint";
        Map<String, Object> map = new HashMap<>(16);
        map.put("applicationName", applicationName);
        map.put("agentId", agentId);
        map.put("password", "admin");
        HttpRequest request = HttpUtil.createGet(uri);
        request.setMethod(Method.GET);
        request.form(map);
        request.setConnectionTimeout(2000);
        try (HttpResponse execute = request.execute()) {
            if (!execute.isOk()) {
                throw new RuntimeException("status:" + execute.getStatus() + "\tres:" + execute.body());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 通过应用名称获取agent列表，带时间
     *
     * @param applicationName 应用名称
     * @param st              开始时间
     * @param ed              结束时间
     * @return 结果
     */
    @Override
    public List<AgentVO> agentList(String applicationName, LocalDateTime st, LocalDateTime ed) {
        List<AgentVO> result = new ArrayList<>();
        long end = ed.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
        long start = st.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
        String uri = baseUri() + "/getAgentList.pinpoint";
        Map<String, Object> map = new HashMap<>(16);
        map.put("application", applicationName);
        map.put("from", start);
        map.put("to", end);
        HttpRequest request = HttpUtil.createGet(uri);
        request.setMethod(Method.GET);
        request.form(map);
        request.setConnectionTimeout(2000);
        try (HttpResponse execute = request.execute()) {
            if (execute.isOk()) {
                String res = new String(execute.body().getBytes(), StandardCharsets.UTF_8);
                cn.hutool.json.JSONObject object = JSONUtil.parseObj(res, true);
                for (String key : object.keySet()) {
                    cn.hutool.json.JSONArray arr = object.getJSONArray(key);
                    for (int i = 0; i < arr.size(); i++) {
                        AgentVO vo = new AgentVO();
                        cn.hutool.json.JSONObject obj = arr.getJSONObject(i);
                        Integer code = obj.getByPath("status.state.code", Integer.class);
                        String agentId = obj.getByPath("status.agentId", String.class);
                        vo.setCode(code);
                        vo.setAgentId(agentId);
                        result.add(vo);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 通过应用名称获取agent列表，不带时间
     *
     * @param applicationName 应用名称
     * @return 结果
     */
    @Override
    public List<AgentVO> agentList(String applicationName) {
        List<AgentVO> result = new ArrayList<>();
        String uri = baseUri() + "/getAgentList.pinpoint";
        Map<String, Object> map = new HashMap<>(16);
        map.put("application", applicationName);
        HttpRequest request = HttpUtil.createGet(uri);
        request.setMethod(Method.GET);
        request.form(map);
        request.setConnectionTimeout(2000);
        try (HttpResponse execute = request.execute()) {
            if (execute.isOk()) {
                String res = new String(execute.body().getBytes(), StandardCharsets.UTF_8);
                cn.hutool.json.JSONObject object = JSONUtil.parseObj(res, true);
                for (String key : object.keySet()) {
                    cn.hutool.json.JSONArray arr = object.getJSONArray(key);
                    for (int i = 0; i < arr.size(); i++) {
                        AgentVO vo = new AgentVO();
                        cn.hutool.json.JSONObject obj = arr.getJSONObject(i);
                        Integer code = obj.getByPath("status.state.code", Integer.class);
                        String agentId = obj.getByPath("status.agentId", String.class);
                        vo.setCode(code);
                        vo.setAgentId(agentId);
                        result.add(vo);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

}
